﻿--[[
	Persistent Data for RaidTracker, Saved Variables

	Aribrary definitions to keep things clear on identifiers:
		Group   - the set of 5 or less members in a party, raid or BG, or any set of members in any type of group
		Party   - group of 5 or less that is not in a raid and is using the Party UI
		Raid    - a set of one or more groups that has formed into a raid and is using the Raid UI
		BG      - a raid that is a pvp raid or party and is using the PvP UI (BattleGroup)

		Rarity - the quality or rarity of an item, quality and rarity are used interchangably
				 rarity is chosen for identifier because using word quality by self could be ambiguous
			[RarityID]  1:poor, 2:common, 3:uncommon, 4:rare, 5:epic, 6:legendary, 7:artifact
]]


RaidTracker = LibKarma:GetInstance( "RaidTracker", {

} );

RaidTracker:RegisterDB( "KARaidTrackerDB", {
	Options = {
		AutoRaid = 2,                   -- [0:off|1:log|2:create]  logging raid if a raid
		AutoParty = 2,					-- [0:off|1:log|2:create]  logging raid if a party
		AutoSolo = 1,					-- [0:off|1:log|2:create]  logging raid if solo
		AutoBattlegroup = 0,			-- [0:off|1:log|2:create]  logging raid if a battelgroup
		AutoArena = 0,					-- [0:off|1:log|2:create]  logging raid if a battelgroup
		AutoEvent = 2,                  -- [0:off|1:onMouseover|2:onBossKill]  when to check for auto boss
		AutoZone = 1,                   -- [0|1]  automatically determine zone
		CostAsk = 0,                    -- [RarityID|0:off]  minimum item rarity to ask for dkp cost for items (ask user)
		CostGet = 0,                	-- [RarityID|0:off]  minimum item rarity to get for dkp cost for items (get from plugin)
		DebugMode = 0,                  -- [0|1]  Debug mode for development
		ExportFormat = 2,               -- [0:MLdkp1.1/EQdkpPlugin|1:EQdkpUnknown|2:MLdkp1.5]  Export format for dkp
		GroupItems = 3,                 -- [RarityID|0:off]  maximum item rarity to group items by count instead of log individualy
		LogAttendees = 1,               -- [0:off|1:onLoot|2:onBoss]  log attendees in database on specified event
		LogGuild = 0,                   -- [0|1]  Snapshots the guildroster on bosskill, adds all online guildies as attendees
		LogWipe	= 1,					-- [0:off|1:always|2..9:onAsk(10-80%)] ask if the group dies if it is a wipe, if all are dead it will not ask
		LevelMax = 80,                  -- If player lvl is maxlevel it will not be exported to mldkp
		LogItems = 2,                   -- [RarityID|0:off]
		SaveExtPlayer = 1,				-- on/off - save race, class and level
		SaveTooltips = 0,               -- on/off - save tooltips of items
		TimeFormat = 0,                 -- on/off - Use 24h time format
		Timezone = 0,					-- logging players timezone offset
		-- Internal flags
		Color = "|c0076dfff",			-- optional text accent color
		ItemIcon = "INV_Misc_Gear_08",	-- default item icon, used if none exists
		ItemOptionsSelected = nil,      -- last selected item option
		CurrentRaid = nil,				-- raid id of raid currently beign logged
		BossName = "Trash mob",         -- just the name of the boss
		EventLast = 0,					-- timer to not initiate a new event
		EventCooldown = 120,			-- [seconds]  how long should trash mob's ignored after a boss kill
		WipeLast = 0,					-- timer to not bug if was a wipe again
		WipeCooldown = 150,             -- how long should death be ignored after a wipe count (seconds)
		VersionCopy = 0,                -- [0|1]  Is imported CT_RaidTracker, one time thing very first load only
		VersionFix = 20,                -- [0...]  Current database version level
	},
	SortOptions = {
		method = "name",
		way = "asc",
		itemsmethod = "looted",
		itemsway = "asc",
		itemsfilter = 1,
		playeritemsfilter = 1,
		playeritemsmethod = "name",
		playeritemsway = "asc",
		playerraidway = "desc",
		itemhistorymethod = "name",
		itemhistoryway = "asc",
	},
	ItemOptions = {
		-- BC
		[29434] = { id = 29434, name = "Badge of Justice",        Track = 0, Group = 1, CostGet = 0, CostAsk = 0, quality = 4, },
		[22450] = { id = 29434, name = "Void Crystal",            Track = 0, Group = 1, CostGet = 0, CostAsk = 0, quality = 4, },
		--   TK - Kael'thas
		[30312] = { id = 30311, name = "Warp Slicer",             Track = 0, Group = 1, CostGet = 0, CostAsk = 0, quality = 5, },
		[30312] = { id = 30312, name = "Infinity Blade",          Track = 0, Group = 1, CostGet = 0, CostAsk = 0, quality = 5, },
		[30313] = { id = 30313, name = "Staff of Disintegration", Track = 0, Group = 1, CostGet = 0, CostAsk = 0, quality = 5, },
		[30314] = { id = 30314, name = "Phaseshift Bulwark",      Track = 0, Group = 1, CostGet = 0, CostAsk = 0, quality = 5, },
		[30316] = { id = 30316, name = "Devastation",			  Track = 0, Group = 1, CostGet = 0, CostAsk = 0, quality = 5, },
		[30317] = { id = 30317, name = "Cosmic Infuser",          Track = 0, Group = 1, CostGet = 0, CostAsk = 0, quality = 5, },
		[30318] = { id = 30318, name = "Netherstrand Longbow",    Track = 0, Group = 1, CostGet = 0, CostAsk = 0, quality = 5, },
		[30319] = { id = 30319, name = "Nether Spike",            Track = 0, Group = 1, CostGet = 0, CostAsk = 0, quality = 5, },
		--   BT - Naj'entus
		[32408] = { id = 32408, name = "Naj'entus Spine",         Track = 0, Group = 1, CostGet = 0, CostAsk = 0, quality = 2, },
		-- WotLK
		[40752] = { id = 40752, name = "Emblem of Heroism",       Track = 0, Group = 1, CostGet = 0, CostAsk = 0, quality = 4, },
		[40753] = { id = 40753, name = "Emblem of Valor",         Track = 0, Group = 1, CostGet = 0, CostAsk = 0, quality = 4, },
	},
	Zones = RaidTracker_Zones,
	Bosses = RaidTracker_BossTriggers,
	Online = { },
	Log = { },
} );


function RaidTracker:RunVersionFix( frame )
	local db = self._store.db;

	-- fixup for missing db updates
	if db.Options.VersionCopy == 1 and not rawget(db.Options, "VersionFix") then db.Options.VersionFix = 7 end
	db.Options.VersionFix = db.Options.VersionFix	-- forces the database to store the default value

	if db.Options.VersionCopy == 0 then
		if CT_RaidTracker_RaidLog then
			self:Print("Importing CT_RaidTracker database.")
			self:ImportToDB( "Options", _G, "CT_RaidTracker_", false, false, {
				"VersionFix",CurrentRaid="GetCurrentRaid","MinQuality","AutoRaidCreation","GroupItems",DkpValue="GetDkpValue","OldFormat",
				"AutoBoss","AutoBossBoss","AutoZone",CostAsk="AskCosts","Timezone","TimeSync",DebugMode="DebugFlag", } );
			self:ImportToDB( nil, _G, "CT_RaidTracker_", true, false, { "Options","ItemOptions","SortOptions", } );
			self:ImportToDB( nil, _G, "CT_RaidTracker_", false, false, { "RaidLog","CustomZoneTriggers","Online", } );
		end
		db.Options.VersionCopy = 1;
	end

	local o = self._options;
	local log = db.RaidLog or db.Log;

	-- 1 - 7 roll-up
	if o.VersionFix >= 0 and o.VersionFix <= 6 then						self:Debug("VersionFix", 1,2,3,4,5,6,7);
		for k,v in pairs(log) do
			if not v.PlayerInfos then v.PlayerInfos = { } end
			if v.Notes then
				for k1,v1 in pairs(v.Notes) do
					if not v.PlayerInfos[k1] then v.PlayerInfos[k1] = { } end
					if not v.PlayerInfos[k1].note then v.PlayerInfos[k1].note = v1; end
				end
				v.Notes = nil;
			end
			if v.BossKills then
				for k1,v1 in pairs(v.BossKills) do
					if type(v1) ~= "table" then
						tinsert(v.BossKills, { boss = k2, time = v2, attendees = {}, } );
						v.BossKills[k1] = nil;
					end
				end
				for k1,v1 in pairs(v.BossKills) do
					if type(v1.time) == "table" then
						v1.boss = v1.time.boss;
						v1.attendees = v1.time.attendees;
						v1.time = v1.time.time;
					end
				end
			end
		end
		if not o.ExportFormat then
			o.MLdkp = o.MLdkp or 1; o.Wipe = o.Wipe or 1; o.WipePercent = o.WipePercent or 1; o.NextBoss = o.NextBoss or 1
			o.ExportFormat = (o.MLdkp == 1) and 2 or ((o.OldFormat and o.OldFormat == 1) and 0 or 1);
			o.MLdkp = nil; o.OldFormat = nil;
		end
		o.VersionFix = 7;
	end;
	-- 8 - 19 roll-up
	if o.VersionFix >= 7 and o.VersionFix <= 18 then					self:Debug("VersionFix", 8,9,10,11,12,13,14,15,16,17,18,19);
		if o.AutoGroup == 1 then o.LogGroup = nil end
		if o.AutoRaidCreation == 1 then o.AutoRaidCreation = nil end
		if o.LogBattlefield == 1 and not rawget(o, AutoArena) then o.AutoArena = 1 end
		self:ImportRenameDB( _G, { RaidTrackerAceDB="KARaidTrackerAceDB", } )
		self:ImportToDB( nil, nil, nil, false, true, { Log="RaidLog", } ); log = db.Log;
		self:ImportToDB( nil, nil, nil, true, true, { ZoneTriggers="CustomZoneTriggers", } );
		self:ImportToDB( nil, nil, nil, true, true, { Zones="ZoneTriggers", } );
		self:ImportRenameDB( o, { CurrentRaid="GetCurrentRaid", SavePlayerDetail="SaveExtendedPlayerInfo", CostAsk="AskCosts", DkpValue="GetDkpValue",
			WipeAsk="Wipe",	WipeCooldown="WipeCoolDown", AutoParty="LogGroup", AutoRaid="AutoRaidCreation", BossName="AutoBossBoss",
			BossCooldown="AutoBossChangeMinTime", LogGuild="GuildSnapshot", LogItems="MinQuality", AutoBattlegroup="LogBattlefield", "MaxLevel",
			"DebugFlag", "DebugMode", "24hFormat", "AutoGroup", "TimeOffsetStatus", "TimeZone", "SaveTooltips", "TimeSync", "BossAsk", "NextBoss",
			"WipePercent", "LevelMax", "TimeFormat", "AutoEvent", "AutoBoss",  } );
		if o.WipeAsk and o.WipePercent and o.WipeAsk > 0 then o.WipeAsk = min(floor(o.WipePercent*10+2),9); end
		self:ImportRenameDB( o, { CostGet="DkpValue", SaveExtPlayer="SavePlayerDetail", LogWipe="WipeAsk", EventCooldown="BossCooldown", } );
		self:ImportRenameDB( db.SortOptions, { itemsfilter="itemfilter", itemsmethod="itemmethod", itemsway="itemway",
			playeritemsfilter="playeritemfilter", playeritemsmethod="playeritemmethod", playeritemsway="playeritemway", } );
		local t = { Track="status", CostAsk="askcosts", CostGet="costgrabbing", Group="group", }
		for k,v in pairs(db.ItemOptions) do
			if type(v) == "table" then
				self:ImportRenameDB( v, t );
				tremove( db.ItemOptions, k );
				db.ItemOptions[v.id] = v;
			end
		end
		for k, v in pairs(db.Zones) do
			if RaidTracker_Zones[k] then db.Zones[k] = nil end
			if type(v) == "string" then db.Zones[k] = { id=0, pid=0, rel=0, name=v } end
		end
		local t = { bossnext="nextboss", Events="BossKills", Key="key", Zone="zone", Wipes="wipes", Players="PlayerInfos", }
		for k,v in pairs(log) do
			self:ImportRenameDB( v, t );
			if not v.Players then v.Players = { } end
			if v.zone == "Tempest Keep: The Eye" then v.zone = "Tempest Keep" end
			if v.Key and type(v.Key) ~= "number" then v.Key = self:GetTimestamp(v.Key) end
			if v.End and type(v.End) ~= "number" then v.End = self:GetTimestamp(v.End) end
			for k1,v1 in pairs(v) do
				if type(v1) == "table" then
					for k2,v2 in pairs(v1) do
						if type(v2) == "table" and v2.time and type(v2.time)~="number" then v2.time = self:GetTimestamp(v2.time) end
					end
				end
			end
			for k1,v1 in pairs(v.Loot) do
				v1.id = k1
				v1.thisitemid = nil
				if v1.player == "disenchanted" then v1.player = "disenchant" end
				local c = c and tonumber(v1.item.count);	v1.item.count = (c==0 or c==1) and nil or c
				local c = v1.attendees;	if c and (type(c)~="table" or #c==0) then v1.attendees = nil end
				if v1.item.tooltip then v1.item.tooltip = nil end
			end
			for k1,v1 in pairs(v.Events) do
				local c = v1.attendees;	if c and (type(c)~="table" or #c==0) then v1.attendees = nil end
			end
		end
		o.VersionFix = 19;
	end;
	if o.VersionFix >= 19 and o.VersionFix <= 20 then						self:Debug("VersionFix", 21);
		local t = { zone="Zone", realm="Realm", key="Key", }
		for k,v in pairs(log) do
			self:ImportRenameDB( v, t );
			v.id = v.id or v.instanceid
			if v.id or v.reset or v.zone then
				if not v.Inst then v.Inst = { } end
				v.Inst.id = v.Inst.id or v.id
				v.Inst.name = v.Inst.name or v.zone
				v.Inst.reset = v.Inst.reset or v.reset
				v.id = nil; v.reset = nil; v.instanceid = nil;
			end
			for k1,v1 in pairs(v.Loot) do
				local c = v1.attendees;	if c and (type(c)~="table" or #c==0 or type(c[1])~="string") then v1.attendees = nil end
			end
			for k1,v1 in pairs(v.Events) do
				local c = v1.attendees;	if c and (type(c)~="table" or #c==0 or type(c[1])~="string") then v1.attendees = nil end
			end
		end
		o.VersionFix = 21;
	end;

end
